'use strict';

var catalogModel = require('@backstage/catalog-model');
var KcAdminClient = require('@keycloak/keycloak-admin-client');
var lodash = require('lodash');
var uuid = require('uuid');
var backendTasks = require('@backstage/backend-tasks');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
  if (e && typeof e === 'object' && 'default' in e) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var KcAdminClient__default = /*#__PURE__*/_interopDefaultCompat(KcAdminClient);
var uuid__namespace = /*#__PURE__*/_interopNamespaceCompat(uuid);

const KEYCLOAK_ID_ANNOTATION = "keycloak.org/id";
const KEYCLOAK_REALM_ANNOTATION = "keycloak.org/realm";
const KEYCLOAK_ENTITY_QUERY_SIZE = 100;

const readProviderConfig = (id, providerConfigInstance) => {
  var _a, _b;
  const baseUrl = providerConfigInstance.getString("baseUrl");
  const realm = (_a = providerConfigInstance.getOptionalString("realm")) != null ? _a : "master";
  const loginRealm = (_b = providerConfigInstance.getOptionalString("loginRealm")) != null ? _b : "master";
  const username = providerConfigInstance.getOptionalString("username");
  const password = providerConfigInstance.getOptionalString("password");
  const clientId = providerConfigInstance.getOptionalString("clientId");
  const clientSecret = providerConfigInstance.getOptionalString("clientSecret");
  const userQuerySize = providerConfigInstance.getOptionalNumber("userQuerySize");
  const groupQuerySize = providerConfigInstance.getOptionalNumber("groupQuerySize");
  if (clientId && !clientSecret) {
    throw new Error(`clientSecret must be provided when clientId is defined.`);
  }
  if (clientSecret && !clientId) {
    throw new Error(`clientId must be provided when clientSecret is defined.`);
  }
  if (username && !password) {
    throw new Error(`password must be provided when username is defined.`);
  }
  if (password && !username) {
    throw new Error(`username must be provided when password is defined.`);
  }
  const schedule = providerConfigInstance.has("schedule") ? backendTasks.readTaskScheduleDefinitionFromConfig(
    providerConfigInstance.getConfig("schedule")
  ) : void 0;
  return {
    id,
    baseUrl,
    loginRealm,
    realm,
    username,
    password,
    clientId,
    clientSecret,
    schedule,
    userQuerySize,
    groupQuerySize
  };
};
const readProviderConfigs = (config) => {
  const providersConfig = config.getOptionalConfig(
    "catalog.providers.keycloakOrg"
  );
  if (!providersConfig) {
    return [];
  }
  return providersConfig.keys().map((id) => {
    const providerConfigInstance = providersConfig.getConfig(id);
    return readProviderConfig(id, providerConfigInstance);
  });
};

const noopGroupTransformer = async (entity, _user, _realm) => entity;
const noopUserTransformer = async (entity, _user, _realm, _groups) => entity;
const sanitizeEmailTransformer = async (entity, _user, _realm, _groups) => {
  entity.metadata.name = entity.metadata.name.replace(/[^a-zA-Z0-9]/g, "-");
  return entity;
};

const parseGroup = async (keycloakGroup, realm, groupTransformer) => {
  var _a, _b;
  const transformer = groupTransformer != null ? groupTransformer : noopGroupTransformer;
  const entity = {
    apiVersion: "backstage.io/v1beta1",
    kind: "Group",
    metadata: {
      name: keycloakGroup.name,
      annotations: {
        [KEYCLOAK_ID_ANNOTATION]: keycloakGroup.id,
        [KEYCLOAK_REALM_ANNOTATION]: realm
      }
    },
    spec: {
      type: "group",
      profile: {
        displayName: keycloakGroup.name
      },
      // children, parent and members are updated again after all group and user transformers applied.
      children: (_b = (_a = keycloakGroup.subGroups) == null ? void 0 : _a.map((g) => g.name)) != null ? _b : [],
      parent: keycloakGroup.parent,
      members: keycloakGroup.members
    }
  };
  return await transformer(entity, keycloakGroup, realm);
};
const parseUser = async (user, realm, keycloakGroups, userTransformer) => {
  const transformer = userTransformer != null ? userTransformer : noopUserTransformer;
  const entity = {
    apiVersion: "backstage.io/v1beta1",
    kind: "User",
    metadata: {
      name: user.username,
      annotations: {
        [KEYCLOAK_ID_ANNOTATION]: user.id,
        [KEYCLOAK_REALM_ANNOTATION]: realm
      }
    },
    spec: {
      profile: {
        email: user.email,
        ...user.firstName || user.lastName ? {
          displayName: [user.firstName, user.lastName].filter(Boolean).join(" ")
        } : {}
      },
      memberOf: keycloakGroups.filter((g) => {
        var _a;
        return (_a = g.members) == null ? void 0 : _a.includes(user.username);
      }).map((g) => g.entity.metadata.name)
    }
  };
  return await transformer(entity, user, realm, keycloakGroups);
};
function* traverseGroups(group) {
  var _a;
  yield group;
  for (const g of (_a = group.subGroups) != null ? _a : []) {
    g.parent = group.name;
    yield* traverseGroups(g);
  }
}
async function getEntities(entities, config, entityQuerySize = KEYCLOAK_ENTITY_QUERY_SIZE) {
  const rawEntityCount = await entities.count({ realm: config.realm });
  const entityCount = typeof rawEntityCount === "number" ? rawEntityCount : rawEntityCount.count;
  const pageCount = Math.ceil(entityCount / entityQuerySize);
  const entityPromises = Array.from(
    { length: pageCount },
    (_, i) => entities.find({
      realm: config.realm,
      max: entityQuerySize,
      first: i * entityQuerySize
    })
  );
  const entityResults = (await Promise.all(entityPromises)).flat();
  return entityResults;
}
const readKeycloakRealm = async (client, config, options) => {
  const kUsers = await getEntities(
    client.users,
    config,
    options == null ? void 0 : options.userQuerySize
  );
  const rawKGroups = await getEntities(
    client.groups,
    config,
    options == null ? void 0 : options.groupQuerySize
  );
  const flatKGroups = rawKGroups.reduce((acc, g) => {
    const newAcc = acc.concat(...traverseGroups(g));
    return newAcc;
  }, []);
  const kGroups = await Promise.all(
    flatKGroups.map(async (g) => {
      g.members = (await client.groups.listMembers({
        id: g.id,
        max: options == null ? void 0 : options.userQuerySize,
        realm: config.realm
      })).map((m) => m.username);
      return g;
    })
  );
  const parsedGroups = await kGroups.reduce(
    async (promise, g) => {
      const partial = await promise;
      const entity = await parseGroup(
        g,
        config.realm,
        options == null ? void 0 : options.groupTransformer
      );
      if (entity) {
        const group = {
          ...g,
          entity
        };
        partial.push(group);
      }
      return partial;
    },
    Promise.resolve([])
  );
  const parsedUsers = await kUsers.reduce(
    async (promise, u) => {
      const partial = await promise;
      const entity = await parseUser(
        u,
        config.realm,
        parsedGroups,
        options == null ? void 0 : options.userTransformer
      );
      if (entity) {
        const user = { ...u, entity };
        partial.push(user);
      }
      return partial;
    },
    Promise.resolve([])
  );
  const groups = parsedGroups.map((g) => {
    var _a, _b, _c, _d, _e;
    const entity = g.entity;
    entity.spec.members = (_b = (_a = g.entity.spec.members) == null ? void 0 : _a.map(
      (m) => {
        var _a2;
        return (_a2 = parsedUsers.find((p) => p.username === m)) == null ? void 0 : _a2.entity.metadata.name;
      }
    )) != null ? _b : [];
    entity.spec.children = (_d = (_c = g.entity.spec.children) == null ? void 0 : _c.map(
      (c) => {
        var _a2;
        return (_a2 = parsedGroups.find((p) => p.name === c)) == null ? void 0 : _a2.entity.metadata.name;
      }
    )) != null ? _d : [];
    entity.spec.parent = (_e = parsedGroups.find(
      (p) => p.name === entity.spec.parent
    )) == null ? void 0 : _e.entity.metadata.name;
    return entity;
  });
  return { users: parsedUsers.map((u) => u.entity), groups };
};

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const withLocations = (baseUrl, realm, entity) => {
  var _a;
  const kind = entity.kind === "Group" ? "groups" : "users";
  const location = `url:${baseUrl}/admin/realms/${realm}/${kind}/${(_a = entity.metadata.annotations) == null ? void 0 : _a[KEYCLOAK_ID_ANNOTATION]}`;
  return lodash.merge(
    {
      metadata: {
        annotations: {
          [catalogModel.ANNOTATION_LOCATION]: location,
          [catalogModel.ANNOTATION_ORIGIN_LOCATION]: location
        }
      }
    },
    entity
  );
};
class KeycloakOrgEntityProvider {
  constructor(options) {
    this.options = options;
    __publicField(this, "connection");
    __publicField(this, "scheduleFn");
  }
  static fromConfig(configRoot, options) {
    return readProviderConfigs(configRoot).map((providerConfig) => {
      let taskRunner;
      if (options.scheduler && providerConfig.schedule) {
        taskRunner = options.scheduler.createScheduledTaskRunner(
          providerConfig.schedule
        );
      } else if (options.schedule) {
        taskRunner = options.schedule;
      } else {
        throw new Error(
          `No schedule provided neither via code nor config for KeycloakOrgEntityProvider:${providerConfig.id}.`
        );
      }
      const provider = new KeycloakOrgEntityProvider({
        id: providerConfig.id,
        provider: providerConfig,
        logger: options.logger,
        userTransformer: options.userTransformer,
        groupTransformer: options.groupTransformer
      });
      if (taskRunner !== "manual") {
        provider.schedule(taskRunner);
      }
      return provider;
    });
  }
  getProviderName() {
    return `KeycloakOrgEntityProvider:${this.options.id}`;
  }
  async connect(connection) {
    var _a;
    this.connection = connection;
    await ((_a = this.scheduleFn) == null ? void 0 : _a.call(this));
  }
  /**
   * Runs one complete ingestion loop. Call this method regularly at some
   * appropriate cadence.
   */
  async read(options) {
    var _a, _b;
    if (!this.connection) {
      throw new Error("Not initialized");
    }
    const logger = (_a = options == null ? void 0 : options.logger) != null ? _a : this.options.logger;
    const provider = this.options.provider;
    const { markReadComplete } = trackProgress(logger);
    const kcAdminClient = new KcAdminClient__default.default({
      baseUrl: provider.baseUrl,
      realmName: provider.loginRealm
    });
    let credentials;
    if (provider.username && provider.password) {
      credentials = {
        grantType: "password",
        clientId: (_b = provider.clientId) != null ? _b : "admin-cli",
        username: provider.username,
        password: provider.password
      };
    } else if (provider.clientId && provider.clientSecret) {
      credentials = {
        grantType: "client_credentials",
        clientId: provider.clientId,
        clientSecret: provider.clientSecret
      };
    } else {
      throw new Error(
        `username and password or clientId and clientSecret must be provided.`
      );
    }
    await kcAdminClient.auth(credentials);
    const { users, groups } = await readKeycloakRealm(kcAdminClient, provider, {
      userQuerySize: provider.userQuerySize,
      groupQuerySize: provider.groupQuerySize,
      userTransformer: this.options.userTransformer,
      groupTransformer: this.options.groupTransformer
    });
    const { markCommitComplete } = markReadComplete({ users, groups });
    await this.connection.applyMutation({
      type: "full",
      entities: [...users, ...groups].map((entity) => ({
        locationKey: `keycloak-org-provider:${this.options.id}`,
        entity: withLocations(provider.baseUrl, provider.realm, entity)
      }))
    });
    markCommitComplete();
  }
  schedule(taskRunner) {
    this.scheduleFn = async () => {
      const id = `${this.getProviderName()}:refresh`;
      await taskRunner.run({
        id,
        fn: async () => {
          var _a;
          const logger = this.options.logger.child({
            class: KeycloakOrgEntityProvider.prototype.constructor.name,
            taskId: id,
            taskInstanceId: uuid__namespace.v4()
          });
          try {
            await this.read({ logger });
          } catch (error) {
            logger.error("Error while syncing Keycloak users and groups", {
              // Default Error properties:
              name: error.name,
              message: error.message,
              stack: error.stack,
              // Additional status code if available:
              status: (_a = error.response) == null ? void 0 : _a.status
            });
          }
        }
      });
    };
  }
}
function trackProgress(logger) {
  let timestamp = Date.now();
  let summary;
  logger.info("Reading Keycloak users and groups");
  function markReadComplete(read) {
    summary = `${read.users.length} Keycloak users and ${read.groups.length} Keycloak groups`;
    const readDuration = ((Date.now() - timestamp) / 1e3).toFixed(1);
    timestamp = Date.now();
    logger.info(`Read ${summary} in ${readDuration} seconds. Committing...`);
    return { markCommitComplete };
  }
  function markCommitComplete() {
    const commitDuration = ((Date.now() - timestamp) / 1e3).toFixed(1);
    logger.info(`Committed ${summary} in ${commitDuration} seconds.`);
  }
  return { markReadComplete };
}

exports.KeycloakOrgEntityProvider = KeycloakOrgEntityProvider;
exports.noopGroupTransformer = noopGroupTransformer;
exports.noopUserTransformer = noopUserTransformer;
exports.sanitizeEmailTransformer = sanitizeEmailTransformer;
//# sourceMappingURL=KeycloakOrgEntityProvider-Bp6QW9Pi.cjs.js.map
